////////////////////////////////////////////////////////////////////////////////
//
// Date:       Feb. 27, 2007
// Module Name:    recoder_ctrl.v
// Description:    Implement a wishbone controller to control the recoder
//            function.  The module accepts Play/Stop and preset start
//            location.  Sample rate is adjusted by a parameter
//
// Revision:
// Revision 0.01 - File Created
//
// File format:    This file has been formated to use tabstop of 4
//
// Available Command:
//
// Development platform: Spartan-3E Starter kit
//
// Copyright (C ) 2007, Rick Huang
//   
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// 
////////////////////////////////////////////////////////////////////////////////

module ctrl_FSM
        (
        BTN_W, 
        reset,      // BTN_S
        BTN_E, 
        clear,      // Control input 
        ui_busy, ui_record,              // LED output 
        main_clk,                        // Main system clock 100 MHz 
        spi_busy,
        wADR_O, wSTB_O, wDAT_O, wDAT_I,
        wACK_I, wWE_O,
        led
        );

  // Control input / UI output 
  input          BTN_W;
  input          reset;
  input          BTN_E;
  input          clear;
  output         ui_busy;
  output         ui_record;
  
  // Wishbone bus signal, bus master 
  input          main_clk;
  output  [31:0] wADR_O;         // ADDRESS DATA OUTPUT TO WISHBONE
  output  [31:0] wDAT_O;         // Wishbone WRITE DATA
  input   [31:0] wDAT_I;         // Wishbone READ DATA
  output         wSTB_O;
  output         wWE_O;          // 
  input          wACK_I;
  
  // connection to LEDs
  output  [7:0]  led;
  
  /* Current ADC/DAC state */
  input          spi_busy;
  
  // Flipflops 
  reg            ui_busy;
  reg            ui_record;
  
  reg [7:0] LED_reg = 0;

  // ** Generating the sampling clock *****************************
  // * Source clock frequency is 100Mhz
  // * Generate a 40KHz sampling clock
  parameter SAMPLE_RATE_DIV_CONST    = 16'd2500;
  
  reg [15:0] clk_divider;
  reg sample_reg;

  always @ ( posedge main_clk )
    begin
    // * 1 ms clock generator
    if ( reset ) clk_divider <= 16'd0;
    else
      begin
      if ( clk_divider == SAMPLE_RATE_DIV_CONST )
        begin
        clk_divider <= 16'd0;
        sample_reg <= 1;
        end 
      else
        begin
        clk_divider <= clk_divider + 1;
        sample_reg <= 0;
        end
      end
    end


  // ** Main state machine ****************************************
  // * 
  // *

  reg    [31:0] dat_i_reg;             // register to store data read from wishbone input data bus
  reg    [3:0]  spi_state;
  reg    [4:0]  cmd_state;
  reg    [4:0]  cmd_latch;
  reg    [31:0] wADR_O;                // holds wishbone address data
  reg    [31:0] wDAT_O;                // holds wishbone write bus data
  reg    [3:0]  wbus_state;
  reg    [4:0]  wbus_timeout;
  reg           wSTB_O;
  reg           wWE_O;
  reg    [31:0] ram_addr;
  
  parameter  RAM_ADDR_MAX = 32'h24000000;
  parameter  ADC_GAIN_CTRL = 32'h0011;
      
  parameter CMD_IDLE          = 0,            // Command phase
            CMD_WRITE_8       = 1,
            CMD_WRITE_8a      = 2,
            CMD_WRITE_8b      = 3,
            CMD_READ_FOREVER  = 4,
            CMD_READ_FOREVERa = 5,
            CMD_READ_FOREVERb = 6,
            CMD_READ_FOREVERc = 7;
  
  parameter WBUS_IDLE     = 0,                // Wishbone bus state
            WBUS_RD_START = 1,
            WBUS_RD_ACKED = 2,
            WBUS_WR_START = 3,
            WBUS_WR_ACKED = 4,
            WBUS_ERR      = 5;
    
  parameter  WBUS_TIMEOUT_SET = 5'h1f;

  reg [31:0] long_delay = 0;
  //  reg [7:0] DIAG = 0;

  always @ ( posedge main_clk ) 
    begin
    if ( reset ) 
      begin
      cmd_state <= CMD_IDLE;
      long_delay <= 0;
      end 

    else

      begin
      case ( cmd_state )

        CMD_IDLE: 
          begin
          ui_busy <= 0;
          ui_record <= 0;
          if ( clear )
            begin
            cmd_state <= CMD_IDLE;
            end
          else
            begin          // start with writing 8 32 bit values to SDRAM
            cmd_state          <= CMD_WRITE_8;
            end
          end
          
        CMD_WRITE_8:
          begin
          ram_addr           <= 32'h20000000;
          dat_i_reg          <= 32'h01010101;
          if ( wbus_state == WBUS_IDLE ) cmd_state <= CMD_WRITE_8a;
          end
          
        CMD_WRITE_8a:
          begin
          if ( wbus_state == WBUS_IDLE )
            begin
            wDAT_O          <= dat_i_reg;
            wADR_O          <= ram_addr;
            wbus_state      <= WBUS_WR_START;         // WRITE START COMMAND
            ram_addr        <= ram_addr + 32'd4;      // increment ram_addr for next time
            cmd_state       <= CMD_WRITE_8b;
            end
          end
          
        CMD_WRITE_8b:
          begin
          if ( wbus_state == WBUS_IDLE )
            begin
            if ( ram_addr == 32'h20000020 ) 
              begin
              cmd_state <= CMD_READ_FOREVER;
              end
            else                         
              begin 
              cmd_state <= CMD_WRITE_8a;
              dat_i_reg <= dat_i_reg << 1;  // left shift for next time
              end
            end
          end

/////////////////////////////////////////////////////////////////////////////////////

        CMD_READ_FOREVER:
          begin
          ram_addr  <= 32'h20000000;
          cmd_state <= CMD_READ_FOREVERa;
          end
          
        CMD_READ_FOREVERa:
          begin
          if ( wbus_state == WBUS_IDLE )           // Wait here until WBUS_IDLE
            begin
            wADR_O     <= ram_addr;
            wbus_state <= WBUS_RD_START;           // READ COMMAND
            cmd_state  <= CMD_READ_FOREVERb; 
            end
          end
          
        CMD_READ_FOREVERb:
          begin
          if ( wbus_state == WBUS_IDLE )
            begin
            cmd_state  <= CMD_READ_FOREVERc;
            long_delay <= 32'h00000000;             // set the long waiting counter to zero
            LED_reg    <= dat_i_reg[7:0];          // capture the read data
            if ( ram_addr == 32'h2000001C )           // have we reached the end of our interest? 
              begin
              ram_addr   <= 32'h20000000;
              end
            else
              begin
              ram_addr   <= ram_addr + 32'd4;      // increment RAM address by 4 (data is 32 bits wide)
              end
            end
          end

        CMD_READ_FOREVERc:
          begin
          long_delay <= long_delay + 1;
          if ( long_delay == 32'd25000000 )
            begin
            cmd_state <= CMD_READ_FOREVERa;     // quarter second
            end
          end
/////////////////////////////////////////////////////////////////////////////////////

          
//        CMD_CLEAR:                      // Clear the entire SDRAM
//          begin                         // stay in this state while WBUS_IDLE is clear
//          ui_busy <= 1;
//          wDAT_O <= 32'd0;
//          if ( wbus_state == WBUS_IDLE )
//            begin
//            cmd_state <= CMD_CLEAR1;
//            wbus_state <= WBUS_WR_START;
//            end
//          end
//          
//        CMD_CLEAR1:
//          begin
//          if ( wbus_state == WBUS_IDLE ) 
//            begin
//            wADR_O <= wADR_O + 32'd4;
//            if ( wADR_O == RAM_ADDR_MAX ) cmd_state <= CMD_SET_GAIN;
//            else                          cmd_state <= CMD_CLEAR;
//            end
//          end
//          
//        CMD_SET_GAIN:
//          begin
//          if ( wbus_state == WBUS_IDLE ) 
//            begin
//            wADR_O     <= 32'h0100;
//            wDAT_O     <= ADC_GAIN_CTRL;
//            cmd_state  <= CMD_IDLE;
//            wbus_state <= WBUS_WR_START;
//            end
//          end
/////////////////////////////////////////////////////////////////////////////////////
          
//        CMD_PLAY:
//          begin
//          //Read from the SDRAM and s  end it to DAC
//          ui_busy <= 1;
//          ram_addr <= 32'h20000000;
//          cmd_state <= CMD_PLAY1;
//          end
//          
//        CMD_PLAY1:
//          begin
//          if ( wbus_state == WBUS_IDLE && sample_reg )
//            begin
//            wADR_O <= ram_addr;
//            cmd_state <= CMD_PLAY2;
//            wbus_state <= WBUS_RD_START;
//            end
//          end
//          
//        CMD_PLAY2:
//          begin
//          if ( wbus_state == WBUS_IDLE )
//            begin
//            wADR_O <= 32'h0102;
//            // data_i_reg is ADC data, 14 bit valid, 2's comp
//            wDAT_O[31:16] <= {dat_i_reg[29:18] + 12'h800, 4'b0};
//            wDAT_O[15:0] <= {dat_i_reg[13:2] + 12'h800, 4'b0};
//            wbus_state <= WBUS_WR_START;
//            ram_addr <= ram_addr + 32'd4;
//            if ( ram_addr == RAM_ADDR_MAX )
//              cmd_state <= CMD_IDLE;
//            else
//              cmd_state <= CMD_PLAY1;
//            end
//          end
/////////////////////////////////////////////////////////////////////////////////////
//
//        CMD_REC:
//          begin
//         //Set the ADC gain control
//          ui_busy <= 1;
//          ui_record <= 1;
//          ram_addr <= 32'h20000000;
//          if ( wbus_state == WBUS_IDLE )
//            begin
//            cmd_state <= CMD_REC1;
//            end
//          end
//          
//        CMD_REC1:
//          begin
//          //Read from the ADC and s  end it to SDRAM
//          if ( wbus_state == WBUS_IDLE && !spi_busy )
//            begin
//            wADR_O <= 32'h101;
//            cmd_state <= CMD_REC2;
//            wbus_state <= WBUS_RD_START;
//            end
//          end
//         
//        CMD_REC2:
//          begin
//          if ( wbus_state == WBUS_IDLE && sample_reg )
//            begin
//            wADR_O <= ram_addr;
//            wDAT_O <= dat_i_reg;
//            wbus_state <= WBUS_WR_START;
//            ram_addr <= ram_addr + 32'd4;
//            if ( ram_addr == RAM_ADDR_MAX )
//              cmd_state <= CMD_IDLE;
//            else
//              cmd_state <= CMD_REC1;
//            end
//          end
          
        default:
          cmd_state <= CMD_IDLE;
        endcase

///////////////////////////////////////////////////////////////////////////////////////////////////    
///////////////////////////////////////////////////////////////////////////////////////////////////    
///////////////////////////////////////////////////////////////////////////////////////////////////    
// THIS BIT TALKS TO THE SDRAM
    
      case ( wbus_state )            // WBus runs
        WBUS_RD_START:
          if ( wACK_I == 0 )
            begin
            wSTB_O          <= 1;          // Start a read cycle
            wWE_O           <= 0;         // set write enable off
            wbus_timeout    <= wbus_timeout - 1;
            if ( wbus_timeout == 0 ) 
              begin
              wbus_state <= WBUS_ERR;
              end
            end 
          else
            begin
            wbus_state <= WBUS_RD_ACKED;  // Received a ACK signal
            wSTB_O <= 0;
            dat_i_reg <= wDAT_I;          // this is the data found on the wishbone input data bus
            end
      
        WBUS_RD_ACKED:
          begin
          if ( wACK_I == 0 ) wbus_state <= WBUS_IDLE;
          end
      
        WBUS_WR_START:
          if ( wACK_I == 0 )
            begin
            wSTB_O <= 1;          // Start a write cycle
            wWE_O  <= 1;          // SET WRITE ENABLE
            wbus_timeout <= wbus_timeout - 1;
            if ( wbus_timeout == 0 ) wbus_state <= WBUS_ERR;
            end 
          else
            begin
            wbus_state <= WBUS_WR_ACKED;  // Received a ACK signal
            wSTB_O <= 0;
            wWE_O <= 0;           // CLEAR WRITE ENABLE
            end
      
        WBUS_WR_ACKED:
          begin
          if ( wACK_I == 0 ) wbus_state <= WBUS_IDLE;
          end
          
        WBUS_IDLE:
          begin
          wSTB_O <= 0;
          wbus_timeout <= WBUS_TIMEOUT_SET;
          end
          
        WBUS_ERR:
          begin
          wSTB_O <= 0;
          wbus_state <= WBUS_IDLE;
          end
        endcase
      end
    end

  assign led = (BTN_W) ? {3'b000,cmd_state} : LED_reg;
//  assign led = (BTN_W) ? DIAG : LED_reg;


endmodule
