////////////////////////////////////////////////////////////////////////////////
//
// Create Date:    02/11/07
// Design Name:    DDR SDRAM memory controller
// Module Name:    ddr_sdram32
// Description:    DDR SDRAM memory driver for most DDR-SDRAM 
//				   Hard coded for 4Meg x 16 x 4bank first
// 
// Revision:
// Revision 0.01 - File Created
//
// Development platform: Spartan-3E Starter kit
//
// Copyright (C) 2007, Rick Huang
//   
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// 
// 
////////////////////////////////////////////////////////////////////////////////

module ddr_sdram32(sd_A_O, sd_BA_O, sd_D_IO, 
				sd_RAS_O, sd_CAS_O, sd_WE_O,
				sd_UDM_O, sd_LDM_O, 
				sd_UDQS_IO, sd_LDQS_IO,
				sd_CS_O, sd_CKE_O, sd_CLK_O,
				main_clk, main_clk_n,				/* Main system clock */
				clk90, clk270,
				wADR_I, wSTB_I, wDAT_I, wDAT_O,		/* Wishbone bus */
			    wACK_O, wWE_I, reset_I, wBURST_I, dbug_st);
	// Connection to DDR-SDRAM memory
	output 	[12:0] sd_A_O;
	output 	[1:0]  sd_BA_O;
	inout 	[15:0] sd_D_IO;
	output	sd_RAS_O;
	output  sd_CAS_O;
	output  sd_WE_O;
	output	sd_UDM_O;
	output	sd_LDM_O;
	inout	  sd_UDQS_IO;
	inout	  sd_LDQS_IO;
	output	sd_CS_O;
	output	sd_CKE_O;
	output 	sd_CLK_O;				// Convert to balanced clock outside
	// Main system clock
	input 	main_clk;
	input	  main_clk_n;				// Invert of main clock
	input   clk90;
	input	  clk270;
	// Wishbone bus signal
	input	 [25:0] wADR_I;			// Use A[23:10] for ROW and A[9:0] for Col
	input 	      wSTB_I;
	input	 [31:0] wDAT_I;
	output [31:0] wDAT_O;
	output	      wACK_O;
	input	        wWE_I;
	input	        reset_I;
	inout         dbug_st;
	input	        wBURST_I;				// Signal a burst operation

	// Local data storage
	reg		[5:0]  sd_state;
	reg		[3:0]  init_state;
	reg		[5:0]  wait_count;
	reg		[14:0] init_wait_count;
	reg		[12:0] sd_A_O;
	reg		[1:0]  sd_BA_O;
	reg		[12:0] mode_reg;
	reg		       sd_RAS_O;
	reg		       sd_CAS_O;
	reg		       sd_WE_O;
	reg		       sd_CS_O;
	reg		       sd_CKE_O;
	reg		       wACK_O;

	reg		[9:0]  refresh_counter;
	reg		[3:0]  refresh_queue;	// Number of refresh command to queue
	reg		       refresh_now;
	reg		       refresh_ack;
	
	reg		[31:0] D_rd_reg;
	reg		       DQS_state;
	reg		       DQS_oe;              // 1 for output
	reg		       D_oe;				      	// 1 for output enable
	reg		       data_sample_en;			// Wbus reading data

	reg		[23:0] D_wr_reg1;		// Burst mode buffer 1 - Collect/assemble
	reg		[31:0] D_wr_reg2;		// Burst mode buffer 2 - Sync to Column command
	reg		[31:0] D_wr_reg;		// Data write buffer
	

/***************************************************		
 * SD ram clock source
 ***************************************************/
	assign sd_CLK_O = main_clk_n;		// Using RAW clock 100MHz


/***************************************************		
 * De-duplex of the data path
 * For some reason, for read, the output is CL=2.5, not CL=2
 * Thus, catching of the data is 1/2 a cycle late.
 * Flip the main_clk and main_clk_n, 1/2 cycle delay catched
 * on data_mux_latch
 ***************************************************/
	// Data from SDRAM will be loaded at {data_mux_out[31:0]}
	wire	[31:0] data_mux_out;
	wire	[15:0] iddr_conn;
	wire	[15:0] oddr_conn;


	generate
	genvar i;

		for(i=0;i<16;i=i+1)
		begin : iou
			IDDR2 #(
				.DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1"
				.INIT_Q0(1'b0), 		// Sets initial state of the Q0 output to 1'b0 or 1'b1
				.INIT_Q1(1'b0), 		// Sets initial state of the Q1 output to 1'b0 or 1'b1
				.SRTYPE("SYNC") 		// Specifies "SYNC" or "ASYNC" set/reset
			) IDDR2_inst (
				.Q0(data_mux_out[i]), 				// C0 clock
				.Q1(data_mux_out[i+16]), 			// C1 clock
				.C0(main_clk_n),		
				.C1(main_clk),
				.CE(1'b1),				// Always enabled
				.D(iddr_conn[i]), 		// 1-bit DDR data input
				.R(1'b0), 				// 1-bit reset input
				.S(1'b0) 				// 1-bit set input
			);

			ODDR2 #(
				.DDR_ALIGNMENT("NONE"), // Sets output alignment to "NONE", "C0" or "C1"
				.INIT(1'b0), 			// Sets initial state of the Q output to 1'b0 or 1'b1
				.SRTYPE("SYNC") 		// Specifies "SYNC" or "ASYNC" set/reset
			) ODDR2_inst (
				.Q(oddr_conn[i]),		// 1-bit DDR output data
				.C0(clk90),			// 1-bit clock input
				.C1(clk270), 		// 1-bit clock input
				.CE(1'b1), 				// 1-bit clock enable input
				.D0(D_wr_reg[i]),		// (associated with C0)
				.D1(D_wr_reg[i+16]),	// (associated with C1)
				.R(1'b0), 				// 1-bit reset input
				.S(1'b0)				// 1-bit set input
			);

			IOBUF #(
				.DRIVE(4), 				// Specify the output drive strength
				.IBUF_DELAY_VALUE("0"), // Specify the amount of added input delay
				.IFD_DELAY_VALUE("AUTO"), // Specify the amount of added delay for input register, "AUTO", "0"-"8"
				.IOSTANDARD("DEFAULT"), // Specify the I/O standard
				.SLEW("SLOW") 			// Specify the output slew rate
			) IOBUF_inst (
				.O(iddr_conn[i]), 		// Buffer output
				.IO(sd_D_IO[i]), 		// Buffer inout port (connect directly to top-level port)
				.I(oddr_conn[i]), 		// Buffer input
				.T(~D_oe) 				// 3-state enable input
			);
		end
	endgenerate


	wire sd_UDQS_I;
	wire sd_LDQS_I;
	wire sd_UDQS_O;
	wire sd_LDQS_O;

		ODDR2 #(
			.DDR_ALIGNMENT("NONE"),
			.INIT(1'b0), 		
			.SRTYPE("SYNC") 
		) ODDR2_inst_UDQ (
			.Q(sd_UDQS_O),
			.C0(main_clk),	
			.C1(main_clk_n),
			.CE(1'b1), 	
			.D0(1'b0),	
			.D1(DQS_state),
			.R(1'b0), 		
			.S(1'b0)	
		);

		ODDR2 #(
			.DDR_ALIGNMENT("NONE"),
			.INIT(1'b0), 		
			.SRTYPE("SYNC") 
		) ODDR2_inst_LDQ (
			.Q(sd_LDQS_O),
			.C0(main_clk),	
			.C1(main_clk_n),
			.CE(1'b1), 	
			.D0(1'b0),	
			.D1(DQS_state),
			.R(1'b0), 		
			.S(1'b0)	
		);

		IOBUF #(
			.DRIVE(4), 	
			.IBUF_DELAY_VALUE("0"), 
			.IFD_DELAY_VALUE("AUTO"),
			.IOSTANDARD("DEFAULT"), 
			.SLEW("SLOW") 		
		) IOBUF_inst_UDQ (
			.O(sd_UDQS_I), 
			.IO(sd_UDQS_IO), 
			.I(sd_UDQS_O),
			.T(~DQS_oe) 		
		);

		IOBUF #(
			.DRIVE(4), 	
			.IBUF_DELAY_VALUE("0"), 
			.IFD_DELAY_VALUE("AUTO"),
			.IOSTANDARD("DEFAULT"), 
			.SLEW("SLOW") 		
		) IOBUF_inst_LDQ (
			.O(sd_LDQS_I), 
			.IO(sd_LDQS_IO), 
			.I(sd_LDQS_O),
			.T(~DQS_oe) 		
		);


	assign dbug_st = sd_LDQS_I;

	reg [31:0] data_mux_latch;
	always @ (posedge main_clk_n)
	begin
		data_mux_latch <= data_mux_out;
	end
	
/***************************************************		
 * Data bus flow direction control
 ***************************************************/
	assign wDAT_O = (data_sample_en ? D_rd_reg[31:0] : 32'd0);

	// Mask register
	reg		UDM_reg_O;
	reg		LDM_reg_O;

	assign	sd_UDM_O = UDM_reg_O;
	assign	sd_LDM_O = LDM_reg_O;
/***************************************************		
 * Main tx/rx state machine
 ***************************************************/
	parameter	WAIT_TIME        = 5'd5;	
	parameter	INIT_WAIT        = 15'h3000;
	parameter	INIT_CLK_EN_WAIT = 15'h10;		
	parameter WAIT_CMD_MAX     = 6'b100000;
	parameter	REFRESH_WAIT     = WAIT_CMD_MAX - 6'd7;
	parameter	ACCESS_WAIT      = WAIT_CMD_MAX - 6'd1;		// AKA RAS time
	parameter	CAS_WAIT         = WAIT_CMD_MAX - 6'd3;
	parameter AVG_REFRESH_DUR = 10'd700;		// Average duration between each refresh
  
				/* Main state machine */
	parameter 	SD_IDLE = 0,
				SD_INIT = 1,
				SD_INIT_WAIT = 2,
				SD_PRECHG_ALL = 3,
				SD_PRECHG_ALL1 = 4,
				SD_AUTO_REF = 5,
				SD_AUTO_REF1 = 6,
				SD_AUTO_REF_ACK = 7,
				SD_LD_MODE = 8,
				SD_LD_MODE1 = 9,
				SD_RD_START = 10,
				SD_RD_COL = 11,
				SD_RD_CAS_WAIT = 12,
				SD_RD_LATCH = 13,
				SD_RD_LATCH1 = 14,
				SD_RD_DONE = 15,
				SD_WR_START = 16,
				SD_WR_COL = 17,
				SD_WR_CAS_WAIT = 18,
				SD_WR_LATCH = 19,
				SD_WR_LATCH1 = 20,
				SD_WR_DONE = 21,
				SD_WR_BURST_ST = 30,
				SD_WR_BURST_ST_D1 = 31,
				SD_WR_BURST_ST_NOP = 32,
				SD_WR_BURST_ST_NOP1 = 33,
				SD_WR_BURST_COL = 34,
				SD_WR_BURST_COL_D1 = 35,
				SD_WR_BURST_NOP = 36,
				SD_WR_BURST_NOP_D1 = 37,
				SD_WR_BURST_END = 38
				;
				/* Initialization state machine */
	parameter	SI_START = 0,
				SI_PRECHG = 1,
				SI_LOAD_EX_MODE = 2,
				SI_LOAD_MODE = 3,
				SI_LOAD_MODE2 = 4,
				SI_PRECHG2 = 5,
				SI_AUTO_REF = 6,
				SI_AUTO_REF2 = 7,
				SI_DONE = 8;


always @ (posedge main_clk) 
begin
	if(reset_I) 
	begin
		sd_state <= SD_INIT;
		init_state <= SI_START;
	end else
	begin
		wait_count <= wait_count + 1;

		case (sd_state)
			SD_INIT:						// Initialize, wait until INIT_WAIT
			begin
				case (init_state)
					SI_START:
					begin
						sd_state <= SD_INIT_WAIT;
						init_state <= SI_PRECHG;
						sd_RAS_O <= 1;
						sd_CAS_O <= 1;
						sd_WE_O <= 1;
						sd_CS_O <= 1;
						UDM_reg_O <= 1;	
						LDM_reg_O <= 1;
						sd_CKE_O <= 0;
						init_wait_count <= 16'd0;
					end
					SI_PRECHG:
					begin
						sd_state <= SD_PRECHG_ALL;
						init_state <= SI_LOAD_EX_MODE;
					end
					SI_LOAD_EX_MODE:
					begin
						// Normal operation
						mode_reg <= 13'b0000000000000;
						sd_BA_O <= 2'b01;
						sd_state <= SD_LD_MODE;	
						init_state <= SI_LOAD_MODE;
					end
					SI_LOAD_MODE:
					begin
						// CAS = 2, Reset DLL, Burst = 2, sequential
						mode_reg <= {6'b000010, 3'b010, 1'b0, 3'b001};
						sd_BA_O <= 2'b00;
						sd_state <= SD_LD_MODE;	
						init_state <= SI_LOAD_MODE2;
					end
					SI_LOAD_MODE2:
					begin
						// CAS = 2, NO Reset DLL, Burst = 2, sequential
						mode_reg <= {6'b000000, 3'b010, 1'b0, 3'b001};
						sd_BA_O <= 2'b00;
						sd_state <= SD_LD_MODE;	
						init_state <= SI_PRECHG2;
					end
					SI_PRECHG2:
					begin
						sd_state <= SD_PRECHG_ALL;
						init_state <= SI_AUTO_REF;
					end
					SI_AUTO_REF:
					begin
						sd_state <= SD_AUTO_REF;
						init_state <= SI_AUTO_REF2;
					end
					SI_AUTO_REF2:
					begin
						sd_state <= SD_AUTO_REF;
						init_state <= SI_DONE;
					end
					SI_DONE:
					begin
						init_state <= SI_DONE;
						sd_state <= SD_IDLE;
					end
					default:
						init_state <= SI_START;
				endcase
			end
			// ** Waiting for SDRAM waking up *****************************
			SD_INIT_WAIT:
			begin
				init_wait_count <= init_wait_count + 1;
				if(init_wait_count == INIT_WAIT)
				begin
					sd_state <= SD_INIT;
				end
				if(init_wait_count == INIT_CLK_EN_WAIT)
				begin
					sd_CKE_O <= 1;				// Wake up the SDRAM
				end
			end
			// ** Precharge command ***************************************
			SD_PRECHG_ALL:
			begin
				sd_state <= SD_PRECHG_ALL1;
				sd_RAS_O <= 0;
				sd_CAS_O <= 1;
				sd_WE_O <= 0;
				sd_CS_O <= 0;
				sd_A_O[10] <= 1;				// Command for precharge all
			end
			SD_PRECHG_ALL1:
			begin
				sd_CS_O <= 1;
				sd_RAS_O <= 1;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				sd_state <= SD_IDLE;			// Precharge takes 15nS before next command
			end
			// ** Load mode register **************************************
			SD_LD_MODE:
			begin
				sd_state <= SD_LD_MODE1;
				sd_RAS_O <= 0;
				sd_CAS_O <= 0;
				sd_WE_O <= 0;
				sd_CS_O <= 0;
				sd_A_O[12:0] <= mode_reg;
			end
			SD_LD_MODE1:
			begin
				sd_CS_O <= 1;
				sd_RAS_O <= 1;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;					// Load Mode takes 12nS
				sd_state <= SD_IDLE;			// Add wait if needed
			end
			// ** Auto refresh command ************************************
			SD_AUTO_REF:
			begin
				sd_state <= SD_AUTO_REF1;
				sd_RAS_O <= 0;
				sd_CAS_O <= 0;
				sd_WE_O <= 1;
				sd_CS_O <= 0;
				wait_count <= REFRESH_WAIT;
			end
			SD_AUTO_REF1:
			begin
				sd_CS_O <= 1;					// Issue NOP during wait
				sd_RAS_O <= 1;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				if(wait_count[5] == 1)			// Time up, return to idle
				begin
					sd_state <= SD_AUTO_REF_ACK;	
					refresh_ack <= 1;
				end
			end
			SD_AUTO_REF_ACK:					// Interlocking state
			begin
				sd_state <= SD_IDLE;	
				refresh_ack <= 0;
			end
			// ** Read cycle **********************************************
			SD_RD_START:
			begin
				sd_state <= SD_RD_COL;
				sd_RAS_O <= 0;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				sd_CS_O <= 0;
				sd_BA_O[1:0] <= wADR_I[25:24];
				sd_A_O[12:0] <= wADR_I[23:11];
				wait_count <= ACCESS_WAIT;
				D_oe <= 0;						// Not driving the bus
				DQS_state <= 0;
			end
			SD_RD_COL:
			begin
				if(wait_count[5] != 1)
				begin
					sd_CS_O <= 1;			// NOP command during access wait		
					sd_RAS_O <= 1;
					sd_CAS_O <= 1;
					sd_WE_O <= 1;
				end else
				begin
					sd_CS_O <= 0;				// Access column 
					sd_RAS_O <= 1;
					sd_CAS_O <= 0;
					sd_WE_O <= 1;
					sd_state <= SD_RD_CAS_WAIT;	
					sd_A_O[9:1] <= wADR_I[10:2];
					sd_A_O[10] <= 1;			// Use auto-precharge
					sd_A_O[0] <= 0;
					wait_count <= CAS_WAIT;
					DQS_oe <= 0;			// Set DQS for input
				end
			end
			SD_RD_CAS_WAIT:
			begin								// Wait until DQS signal there is data
				if(wait_count[5] != 1)
				begin
					sd_CS_O <= 1;				// NOP command during access wait		
					sd_RAS_O <= 1;
					sd_CAS_O <= 1;
					sd_WE_O <= 1;
				end else
				begin
					sd_state <= SD_RD_LATCH;
				end
			end
			SD_RD_LATCH:
			begin
				D_rd_reg <= data_mux_latch[31:0];
				wACK_O <= 1;
				data_sample_en <= 1;
				sd_state <= SD_RD_DONE;
			end
			SD_RD_DONE:
			begin
				wACK_O <= 0;
				data_sample_en <= 0;
				sd_state <= SD_IDLE;
				DQS_state <= 0;
				DQS_oe <= 1;				// Set DQS back to output
			end
			// ** Write cycle *********************************************
			SD_WR_START:
			begin							// Open the bank - ACTIVE command
				sd_state <= SD_WR_COL;
				sd_RAS_O <= 0;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				sd_CS_O <= 0;
				sd_BA_O[1:0] <= wADR_I[25:24];
				sd_A_O[12:0] <= wADR_I[23:11];
				D_wr_reg <= wDAT_I;
				wait_count <= ACCESS_WAIT;
				DQS_state <= 0;
			end
			SD_WR_COL:
			begin
				if(wait_count[5] != 1)
				begin
					sd_CS_O <= 1;			// NOP command during access wait		
					sd_RAS_O <= 1;
					sd_CAS_O <= 1;
					sd_WE_O <= 1;
				end else
				begin
					sd_CS_O <= 0;			// Access column 
					sd_RAS_O <= 1;
					sd_CAS_O <= 0;
					sd_WE_O <= 0;
					DQS_oe <= 1;
					sd_state <= SD_WR_CAS_WAIT;	
					sd_A_O[9:1] <= wADR_I[10:2];
					sd_A_O[10] <= 1;		// Use auto-precharge
					sd_A_O[0] <= 0;
				end
			end
			SD_WR_CAS_WAIT:
			begin							// Wait until DQS signal there is data
				sd_state <= SD_WR_LATCH;
				DQS_state <= 1;			// Start with DQS low
				D_oe <= 1;				// Drive the data bus
				sd_CS_O <= 1;
				sd_RAS_O <= 1;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				LDM_reg_O <= 0;
				UDM_reg_O <= 0;
			end
			SD_WR_LATCH:
			begin
				sd_state <= SD_WR_LATCH1;
				DQS_state <= 0;
			end
			SD_WR_LATCH1:
			begin
				sd_state <= SD_WR_DONE;
				DQS_state <= 0;
				wACK_O <= 1;
				UDM_reg_O <= 1;
				LDM_reg_O <= 1;
			end
			SD_WR_DONE:
			begin
				UDM_reg_O <= 1;
				LDM_reg_O <= 1;
				D_oe <= 0;
				wACK_O <= 0;
				sd_state <= SD_IDLE;
			end

			/*
			SD_WR_BURST_ST:		// Start a burst write process
			begin
				if(sd_CLK_O == 1)			// Open the bank - ACTIVE command
				begin
					sd_state <= SD_WR_BURST_ST_D1;
					sd_RAS_O <= 0;
					sd_CAS_O <= 1;
					sd_WE_O <= 1;
					sd_CS_O <= 0;
					sd_BA_O[1:0] <= wADR_I[25:24];
					sd_A_O[12:0] <= wADR_I[23:11];
					wait_count <= (6'b100000 - ACCESS_WAIT);
				end
			end
			SD_WR_BURST_ST_D1:				// Delay 1 cycle
				sd_state <= SD_WR_BURST_ST_NOP;
			SD_WR_BURST_ST_NOP:				// Start NOP command
			begin
				sd_CS_O <= 1;			
				sd_RAS_O <= 1;
				wACK_O <= 1;				// Start of ACK, next cycle, read data
				sd_state <= SD_WR_BURST_ST_NOP1;
			end
			SD_WR_BURST_ST_NOP1:				
			begin
				D_wr_reg1[7:0] <= wDAT_I;	// 1st byte, in cycle mode, there is another 1st byte
				sd_state <= SD_WR_BURST_COL;
			end

			SD_WR_BURST_COL:				// COL address command
			begin
				sd_CS_O <= 0;				// Access column 
				sd_RAS_O <= 1;
				sd_CAS_O <= 0;
				sd_WE_O <= 0;
				sd_state <= SD_WR_BURST_COL_D1;	
				sd_A_O[9:0] <= wADR_I[10:1];
				sd_A_O[10] <= 0;			// No auto-precharge
				D_wr_reg1[15:8] <= wDAT_I;	// 2nd byte
				DQS_state <= 0;				// Preset to low
				D_oe <= 0;					// Allow data output
			end
			SD_WR_BURST_COL_D1:
			begin
				sd_state <= SD_WR_BURST_NOP;	
				D_wr_reg1[23:16] <= wDAT_I;	// 3rd byte
			end
			SD_WR_BURST_NOP:				// NOP command after the COL command
			begin							// Data is latched into SDRAM during NOP 
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				sd_CS_O <= 1;				
				sd_state <= SD_WR_BURST_NOP_D1;	
				D_wr_reg2[23:0] <= D_wr_reg1[23:0];
				D_wr_reg2[31:24] <= wDAT_I;
				D_oe <= 1;					// Allow data output
			end
			SD_WR_BURST_NOP_D1:
			begin
				D_wr_reg1[7:0] <= wDAT_I;	// 1st byte
				DQS_state <= 1;				// Second data
				if(wSTB_I == 1)
					sd_state <= SD_WR_BURST_COL;	// More data is coming
				else
					sd_state <= SD_WR_BURST_END;	// No more data
			end
			SD_WR_BURST_END:				// End with a precharge command
			begin
				sd_CS_O <= 1;				// Access column 
				sd_RAS_O <= 1;
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				DQS_state <= 0;				// Second data
				sd_state <= SD_PRECHG_ALL;	
			end */

		    // Wishbone transfer is done during idle time
			SD_IDLE: 						/* Idle/sleep process */
			begin
				sd_RAS_O <= 1;				// Set for NOP by default
				sd_CAS_O <= 1;
				sd_WE_O <= 1;
				sd_CS_O <= 1;				
				if(init_state != SI_DONE)	// If still in init cycle, go back and work
					sd_state <= SD_INIT;
				else if(wSTB_I && !wWE_I)				// Start of a read command
					sd_state <= SD_RD_START;
				else if(wSTB_I && wWE_I && !wBURST_I)	// Start of a write command
					sd_state <= SD_WR_START;
		//		else if(wSTB_I && wWE_I && wBURST_I)	// Start of burst write
		//			sd_state <= SD_WR_BURST_ST;
				else if(refresh_now)					// Refresh is the last command
					sd_state <= SD_AUTO_REF;
			end

			default:
				sd_state <= SD_IDLE;
		endcase
	end
end

/* Seperate always block for refresh timer */
/* The idea is to queue as many as 8 refresh command as possible if the bus is
 * free */
always @ (posedge main_clk) 
begin
	if(refresh_ack)
	begin
		refresh_now <= 0;
		refresh_queue <= refresh_queue + 4'd1;
	end else
	if(reset_I) 
	begin
		refresh_counter <= 0;
		refresh_queue <= 4'd0;
	end else
	begin
		refresh_counter <= refresh_counter + 1;
		if(refresh_counter == AVG_REFRESH_DUR)
		begin
			refresh_counter <= 0;
			if(refresh_queue != 4'd0)
				refresh_queue <= refresh_queue - 4'd1;
		end
		if(refresh_queue != 4'd7)
			refresh_now <= 1;
	end
end

endmodule
