`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
//
// Date:         2011-10-14
// Module Name:  hello_world.v
// Engineer:     Scott Gravenhorst
// Description:  A state machine that allows SDRAM initialization to complete,
//               then writes eight 32 bit locations with a walking one, then loops 
//               forever while reading these locations consecutively but slowly
//               (one every 1/4 second) while displaying the low byte in the LEDs.
//               It should display a walking one pattern.
// Version:      0.00
//
// This project is based on the SDRAM interface found in Rick Huang's widebus 
// recorder project.  Attribution is found below
//
// Reformat by Scott Gravenhorst
//
// This project runs at a clock rate of 100 MHz.  2 DCMs are used, the first 
// provides 2X clock out from 50 MHz input.  The second DCM receives the first DCM
// CLKX2 output and outputs the phases required by the project (mainly by the SDRAM 
// interface).
//
// As of 2011-10-16, this project works.
//   NOTE: The reset system requires that you hold down the rotary encoder push
//         button and press and release the south button and then release the
//         rotary encoder pushbutton.  Failure to do so will cause unpredictable
//         results (the LEDs will not do what they are supposed to do).
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Date:       Feb. 27, 2007
// Module Name:    Base.v
// Description:    Main wiring block for the recoder project
//
// Revision:
// Revision 0.01 - File Created
//
// File format:    This file has been formmated to use tabstop of 4
//
// User Interface:
//       Button:
//       South  = Stop/reset
//       North  = Clock reset (Usually not required)
//       East   = Play
//       West   = Record
//       Center = Clear memory
//
//       Sample rate and input gain is adjusted in the recoder control
//       block.
//
// Development platform: Spartan-3E Starter kit
//
// Copyright (C) 2007, Rick Huang
//   
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// 
////////////////////////////////////////////////////////////////////////////////

module hello_world(clk_raw, 
      spi_sck, spi_sdi, spi_rom_cs, spi_amp_cs, 
      spi_adc_conv, spi_dac_cs, spi_amp_shdn, spi_dac_clr,
      strataflash_oe, strataflash_ce, strataflash_we,
      strataflash_byte, 
      strataflash_D[15:0], sf_A[24:0], platformflash_oe, 
      led[7:0], switch[3:0],
      btn_north, btn_east, btn_south, btn_west,
      rot_cnt,                                           // rotary pushbutton
      extspi_si, extspi_so, extspi_sck, extspi_ce,
      lcd_e, lcd_rs, lcd_rw,
      aux_o,
      sd_A[12:0], sd_DQ[15:0], sd_BA[1:0],
      sd_CAS, sd_CK_N, sd_CK_P, sd_CKE, 
      sd_CS, sd_LDM, sd_LDQS, sd_RAS, 
      sd_UDM, sd_UDQS, sd_WE, sd_dbg_st,
      );

  input clk_raw;
  // SPI bus
  output        spi_sck;
  output        spi_sdi;      // Note: Referenced from the peripheral!
  output        spi_rom_cs;
  output        spi_amp_cs;
  output        spi_adc_conv;
  output        spi_dac_cs;
  output        spi_amp_shdn;
  output        spi_dac_clr;
  // Strataflash bus
  output        strataflash_oe;
  output        strataflash_ce;
  output        strataflash_we;
  inout  [15:0] strataflash_D;
  output [24:0] sf_A;
  output        platformflash_oe;
  output        strataflash_byte;
  // UI I/O
  output [7:0]  led;
  input  [3:0]  switch;
  input         btn_north;
  input         btn_east;
  input         btn_south;
  input         btn_west;
  input         rot_cnt;
  // MPU connection
  input         extspi_si;
  input         extspi_sck;
  input         extspi_ce;
  output        extspi_so;
  // LCD signals
  output        lcd_e;
  output        lcd_rs;
  output        lcd_rw;
  // Scope probe point
  output [3:0]  aux_o;
  // DDR-SDRAM connection
  output [12:0] sd_A;
  inout  [15:0] sd_DQ;
  output [1:0]  sd_BA;
  output        sd_CAS;
  output        sd_CK_N, sd_CK_P;
  output        sd_CKE;
  output        sd_CS;
  output        sd_LDM, sd_UDM;
  inout         sd_LDQS, sd_UDQS;
  output        sd_RAS;
  output        sd_WE;
  inout         sd_dbg_st;

  // Debug storage
  reg  [31:0]   debug0;
  
  /* Wishbone bus nets - Ref on MASTER side */
  wire [31:0]   wADR_O;
  wire [31:0]   wDAT_O;
  wire [31:0]   wDAT_I;
  wire          wSTB_O;
  wire          wWE_O;
  wire          wACK_I;
  wire          spi_sdo;
  wire          ext_addr_STB;
  reg  [31:0]   reg_read_buf;

  // STRATAFLASH nets
  wire          wSF_ack;
  wire [7:0]    wSF_DAT_I;
  wire          wSF_STB_O;

  // ********* Assign signal block *************** //
  // Tie off the flash enables to allow SPI to work
  assign spi_sdo = strataflash_D[0];  // Pin is shared
  assign strataflash_byte = 0;    // 8 bit mode
  assign platformflash_oe = 0;
  
  // Tie of all SPI 
  assign spi_rom_cs = 1;
  assign spi_amp_shdn = 0;
  assign spi_dac_clr = 1;

  // Extract LCD signal from the strataflash
  // assign strataflash_D[15:1] = {4'b0, LCD_data, 7'b0};  // Not now
  assign lcd_rw = 0;          // Write only to LCD
  assign LCD_data = 0;        // Use up unused pins
  assign lcd_rs = 0;
  assign lcd_e = 0;  


  // ** Invoke the functional block **********************************
  // synchronize reset
  //
  wire clk_buffed;
  wire dll_locked;
  wire clk;
  wire clk_n;
  wire clk_90;
  wire clk_270;

  // ** Clock buffer and generation ***********************************
  // 
  clkgen2 xclk
      (
      .U1_CLKIN_IN( clk_raw ), 
      .U1_CLKIN_IBUFG_OUT( clk_buffed ),    // clk_buffed is not used (50 MHz)
      .U1_RST_IN( btn_north ), 
      .U2_CLK0_OUT( clk ), 
      .U2_CLK180_OUT( clk_n ),
      .U2_CLK90_OUT( clk_90 ), 
      .U2_CLK270_OUT( clk_270 ),
      .U2_LOCKED_OUT( dll_locked )
      );
  
  // ** Sync UI to internal clock ************************************
  reg btn_west_reg, btn_south_reg, btn_east_reg, btn_cnt_reg;

  // Sync to internal clock
  always @ ( posedge clk )
    begin
    btn_west_reg  <= btn_west;
    btn_south_reg <= btn_south;
    btn_east_reg  <= btn_east;
    btn_cnt_reg   <= rot_cnt;        // rotary encoder center pushbutton
    end
  
  // ** FLASH I/O block, map to wishbone address 0x10000000 ***********
  //
  /*
    nor_flash i_sf(.sf_oe_O(strataflash_oe), .sf_ce_O(strataflash_ce),
        .sf_we_O(strataflash_we), 
        .sf_D_IO(strataflash_D), .sf_A_O(sf_A),
        .main_clk(clk),
        .wADR_I(wADR_O[24:0]), .wSTB_I(wSF_STB_O), 
        .wDAT_I(wDAT_O), .wDAT_O(wSF_DAT_I),
          .wACK_O(wSF_ack), .wWE_I(wWE_O));*/

  //assign wSF_STB_O = (wADR_O[31:28] == 4'h1) ? wSTB_O : 0;

  //Finish connecting the signal for now
  assign strataflash_oe = 1;
  assign strataflash_ce = 1;
  assign strataflash_we = 1;
  assign strataflash_D[15:1] = 0;
  assign sf_A = 0;
  
  // ** DDR-SDRAM I/O block, map to wishbone adddress 0x20000000 ******
  //
  wire [31:0] wSD_DAT_I;
  wire wSD_ack;
  wire sd_dbg_st;
  wire wSD_STB_O;
  
  wire sd_CLK_O;

  assign sd_CK_P = clk_n;
  assign sd_CK_N = clk;
  
  ddr_sdram32 m_sd
    (
    .sd_A_O(sd_A), .sd_BA_O(sd_BA), .sd_D_IO(sd_DQ), 
    .sd_RAS_O(sd_RAS), .sd_CAS_O(sd_CAS), 
    .sd_WE_O(sd_WE),
    .sd_UDM_O(sd_UDM), .sd_LDM_O(sd_LDM), 
    .sd_UDQS_IO(sd_UDQS), .sd_LDQS_IO(sd_LDQS),
    .sd_CS_O(sd_CS), .sd_CKE_O(sd_CKE), 
    .sd_CLK_O(sd_CLK_O),
    .main_clk(clk), .main_clk_n(clk_n),
    .clk90(clk_90), .clk270(clk_270),
    .wADR_I(wADR_O[25:0]), .wSTB_I(wSD_STB_O),
    .wDAT_I(wDAT_O), .wDAT_O(wSD_DAT_I),
    .wACK_O(wSD_ack), 
    .wWE_I(wWE_O), 
    .reset_I(btn_south_reg),
    .wBURST_I(0),
    .dbug_st(sd_dbg_st)
    );

  assign wSD_STB_O = (wADR_O[31:28] == 4'h2) ? wSTB_O : 0; 
  

  // ** Small I/O block, map to wishbone adddress 0x00000000 ********
  //
  reg block0_en;
  reg block1_en;
  
  always @ ( posedge clk )
    begin
    if ( wADR_O[31:8] == 24'h000000)  block0_en <= 1;
    else                              block0_en <= 0;

    if ( wADR_O[31:8] == 24'h000001)  block1_en <= 1;
    else                              block1_en <= 0;
    end
  

  // ** Debug display block ********************* 0x00000080 ********
  // Decode address and map the into registers
  // 0x80 - test signals debug0
  // 0x84 - switch input
  reg         wLED_ack;
  wire [31:0] wLED_DAT_I;

  always @ ( posedge clk )
    begin
    if ( block0_en)
      begin
      if ( wSTB_O == 1 && wWE_O == 1)
        begin
        if ( wADR_O[7:0] == 8'h80) debug0 <= wDAT_O;
        end

      if ( wSTB_O == 1 && wWE_O == 0)
        begin
        if ( wADR_O[7:0] == 8'h80) reg_read_buf <= debug0;
        if ( wADR_O[7:0] == 8'h84) 
          reg_read_buf <= {24'd0, switch[3:0], btn_north, btn_east, btn_south, btn_west};
        end

      wLED_ack <= wSTB_O;
      end 
    else if ( wLED_ack == 1) wLED_ack <= 0;
    end

  assign wLED_DAT_I = (wLED_ack == 1) ? reg_read_buf : 32'd0;
  

  // ** SPI/ADC block *************************** 0x00000100 ********
  //
  wire wADC_ack;
  wire [31:0] wADC_DAT_I;
  wire wSTB_adc;
  wire spi_busy;

  spi_adc adc_cont_inst 
      ( 
      .spi_mosi(spi_sdi), .spi_ampcs(spi_amp_cs), 
      .spi_sck(spi_sck), .spi_adcs(spi_adc_conv), 
      .spi_miso(spi_sdo), .spi_daccs(spi_dac_cs),
      .main_clk(clk), 
      .reset(btn_cnt_reg),        
      .wADR_I(wADR_O[2:0]), 
      .wSTB_I(wSTB_adc), 
      .wDAT_I(wDAT_O), 
      .wDAT_O(wADC_DAT_I),
      .wACK_O(wADC_ack), 
      .wWE_I(wWE_O),
      .spi_busy(spi_busy)
      );

  assign wSTB_adc = wSTB_O & block1_en;

  // ** Recoder controller block ***************************************
  // 
  wire ui_busy;
  wire ui_record;
  
  ctrl_FSM ctrl1 
            (
            .reset( btn_south_reg ), 
            .BTN_W( btn_west_reg ), 
            .BTN_E( btn_east_reg ), 
            .clear( btn_cnt_reg ),
            .ui_busy( ui_busy ), 
            .ui_record( ui_record ),
            .main_clk( clk ),                  
            .spi_busy( spi_busy ),
            .wADR_O( wADR_O ), 
            .wSTB_O( wSTB_O ), 
            .wDAT_O( wDAT_O ), 
            .wDAT_I( wDAT_I ),
            .wACK_I( wACK_I ), 
            .wWE_O( wWE_O ),
            .led( led )
            );

  // Use ORed bus structure
  assign wACK_I = wLED_ack   | wSD_ack | wADC_ack;
  assign wDAT_I = wLED_DAT_I | wSD_DAT_I | wADC_DAT_I;

  assign extspi_so = extspi_si;

  // ** Debug signal block ********************************************
  // Use the 4 switch to select which line to observe
//  assign led = (switch == 4'b0000 ? wDAT_O[7:0] : 0) |
//               (switch == 4'b0001 ? wDAT_O[15:8] : 0) |
//               (switch == 4'b0011 ? wDAT_O[23:16] : 0) |
//               (switch == 4'b0010 ? wDAT_O[31:24] : 0) |
//               (switch == 4'b0100 ? wADR_O[7:0]   : 0) |
//               (switch == 4'b0101 ? wADR_O[15:8]  : 0) |
//               (switch == 4'b0111 ? wADR_O[23:16] : 0) |
//               (switch == 4'b0110 ? wADR_O[31:24] : 0) |
//               (switch == 4'b1100 ? wDAT_I[7:0]   : 0) |
//               (switch == 4'b1101 ? wDAT_I[15:8]  : 0) |
//               (switch == 4'b1111 ? wDAT_I[23:16] : 0) |
//               (switch == 4'b1110 ? wDAT_I[31:24] : 0) |
//               (switch == 4'b1111 ? {4'b0,ui_record ,ui_busy, 1'b0, dll_locked }  : 0) ;

  assign aux_o =  {spi_sdi, spi_amp_cs, spi_sck, spi_sdo};

  endmodule
