////////////////////////////////////////////////////////////////////////////////
//
// Create Date:    02/18/07
// Design Name:    ADC on SPI bus glue logic
// Module Name:    adc_spi
// Description:    The glue logic to link the ADC onto the wishbone bus
//                 Add the DAC logic into the mix too
//
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
//
// File format:    This file has been formated to use tabstop of 4
//
// Development platform: Spartan-3E Starter kit
//
// Copyright (C) 2006, Rick Huang
//   
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// 
// 
////////////////////////////////////////////////////////////////////////////////

module spi_adc( 
        spi_mosi, spi_ampcs, spi_sck, 
        spi_adcs, spi_miso,
        spi_daccs,
        main_clk,                         /* Main system clock (100 MHz) */
        reset,
        wADR_I, wSTB_I, wDAT_I, wDAT_O,    /* Wishbone bus */
        wACK_O, wWE_I, spi_busy);
        
  // Connection to SPI bus
  output  spi_mosi;
  output  spi_ampcs;
  output  spi_sck;
  output  spi_adcs;
  output  spi_daccs;
  input   spi_miso;
  
  // Main system clock
  input   main_clk;
  
  // Reset signals
  input   reset;
  
  // Wishbone bus signal
  input  [2:0]   wADR_I;
  input          wSTB_I;
  input  [31:0]  wDAT_I;
  input          wWE_I;
  output [31:0]  wDAT_O;
  output         wACK_O;
  
  // Indicate the SPI bus is busy
  output        spi_busy;

  // Local data storage
  reg            wACK_O;
  reg            spi_ampcs;        // CS for the AMP
  reg            spi_adcs;         // CS for the ADC
  reg            spi_daccs;        // CS for the ADC
  reg            spi_sck;
  reg   [31:0]   wDAT_O_reg;
  reg            wDAT_OE;
  reg            spi_busy;

  // Wishbone bus output en
  assign wDAT_O = wDAT_OE ? wDAT_O_reg : 32'd0;

// ** SPI clock generation ************************************
  reg    [2:0] divider;           // divide by 8 counter  (12.5 MHz)
  reg          clk_loc;
  
  always @ ( posedge main_clk )
    begin
    if ( reset )
      begin
      divider <= 3'd0;
      end
    else
      begin
      divider <= divider + 1;
      if ( divider == 3'd0 )      clk_loc <= 1'b1;
      else                        clk_loc <= 1'b0;
      end
    end


// ** ADC gain amp state machine ******************************

  reg    [31:0] ShiftReg;
  reg    [4:0]  adc_g_state;
  reg    [5:0]  adc_g_bit_cnt;
  reg           dac_chan_sel;
  reg    [15:0] dac_chan2_data;

  assign  spi_mosi = ShiftReg[31];
  wire  [3:0]  dac_cmd = 4'b0011;

  parameter   AG_IDLE = 0,            /* Declare bit states */
        AG_TX_ST = 1,
        AG_TX_HIGH = 2,
        AG_TX_LOW = 3,
        AG_TX_END = 4,
        AG_RX_ST = 5,
        AG_RX_HIGH = 6,
        AG_RX_LOW = 7,
        AG_RX_LOW1 = 17,
        AG_RX_END1 = 8,
        AG_RX_END2 = 9,
        AG_RX_END3 = 10,
        AG_DA_TX_ST = 11,
        AG_DA_TX_ST2 = 12,
        AG_DA_TX_HIGH = 13,
        AG_DA_TX_LOW = 14,
        AG_DA_TX_END1 = 15,
        AG_DA_TX_END2 = 16;

  always @ ( posedge main_clk )
    begin
    if ( reset )
      begin
      adc_g_state <= AG_IDLE;
      spi_ampcs <= 1;
      spi_adcs <= 0;
      spi_daccs <= 1;
      end 
    else
    case ( adc_g_state )
      AG_IDLE:
        begin
        spi_sck <= 0;
        adc_g_bit_cnt <= 0;
        spi_busy <= 0;
        if ((wSTB_I == 1) && (wADR_I == 3'd0) && (wWE_I == 1))
          begin
          if (clk_loc)
            begin
            adc_g_state <= AG_TX_ST;
            wACK_O <= 1;
            end
          end else
        if ((wSTB_I == 1) && (wADR_I == 3'd1) && (wWE_I == 0))
          begin
          adc_g_state <= AG_RX_ST;
          // Pipelined conversion, return the last conversion's data
          wACK_O <= 1;
          wDAT_OE <= 1;
          spi_adcs <= 1;
          end else
        if ((wSTB_I == 1) && (wADR_I == 3'd2) && (wWE_I == 1))
          begin
          adc_g_state <= AG_DA_TX_ST;
          wACK_O <= 1;
          end 
        end
        
      AG_TX_ST:
        begin
        ShiftReg[31:24] <= wDAT_I[7:0];
        wACK_O <= 0;
        spi_ampcs <= 0;
        spi_busy <= 1;
        if ( clk_loc )
          begin
          adc_g_state <= AG_TX_HIGH;
          adc_g_bit_cnt <= adc_g_bit_cnt + 1;
          end
        end
        
      AG_TX_HIGH:
        begin
        spi_sck <= 1;
        if ( clk_loc )
          begin
          if (adc_g_bit_cnt == 8)
            begin
            adc_g_state <= AG_TX_END;
            end else
            begin
            adc_g_state <= AG_TX_LOW;
            ShiftReg[31:1] <= ShiftReg[30:0];
            end
          end
        end
        
      AG_TX_LOW:
        begin
        spi_sck <= 0;
        if ( clk_loc )
          begin
          adc_g_state <= AG_TX_HIGH;
          adc_g_bit_cnt <= adc_g_bit_cnt + 1;
          end
        end
        
      AG_TX_END:
        begin
        spi_sck <= 0;
        if ( clk_loc )
          begin
          spi_ampcs <= 1;
          adc_g_state <= AG_IDLE;
          end
        end
        
      AG_RX_ST:
        begin
        adc_g_state <= AG_RX_HIGH;
        wACK_O <= 0;
        wDAT_OE <= 0;
        spi_busy <= 1;
        end
        
      AG_RX_HIGH:
        begin
        spi_adcs <= 0;
        spi_sck <= 1;
        if (adc_g_bit_cnt == 6'd32)          adc_g_state <= AG_RX_END1;
        else                                adc_g_state <= AG_RX_LOW1;
        end
        
      AG_RX_LOW1:
        begin
        spi_sck <= 0;
        adc_g_state <= AG_RX_LOW;
        end
        
      AG_RX_LOW:
        begin
        spi_sck <= 0;
        ShiftReg[31:1] <= ShiftReg[30:0];
        ShiftReg[0] <= spi_miso;
        adc_g_bit_cnt <= adc_g_bit_cnt + 1;
        adc_g_state <= AG_RX_HIGH;
        end
        
      AG_RX_END1:
        begin
        spi_sck <= 0;
        wDAT_O_reg <= {ShiftReg[30:0], spi_miso};
        adc_g_state <= AG_RX_END2;
        end
        
      AG_RX_END2:
        begin
        spi_sck <= 1;
        adc_g_state <= AG_RX_END3;
        end
        
      AG_RX_END3:
        begin
        spi_sck <= 0;
        adc_g_state <= AG_IDLE;
        end
        
      AG_DA_TX_ST:
        begin
        dac_chan2_data <= wDAT_I[15:0];
        // Select channel 0
        ShiftReg[31:8] <= {dac_cmd, 4'b0000, wDAT_I[31:16]};
        wACK_O <= 0;
        spi_daccs <= 0;
        adc_g_state <= AG_DA_TX_HIGH;
        dac_chan_sel <= 0;        // Channel 0 first
        spi_busy <= 1;
        end
        
      AG_DA_TX_ST2:
        begin
        // Select channel 1
        ShiftReg[31:8] <= {dac_cmd, 4'b0001, dac_chan2_data};
        wACK_O <= 0;
        spi_daccs <= 0;
        adc_g_state <= AG_DA_TX_HIGH;
        dac_chan_sel <= 1;        // Follow by chan 1
        end
        
      AG_DA_TX_HIGH:
        begin
        spi_sck <= 1;
        if ( adc_g_bit_cnt == 6'd23 )
          begin
          if (dac_chan_sel == 0)
            adc_g_state <= AG_DA_TX_END1;
          else
            adc_g_state <= AG_DA_TX_END2;
          end else
          adc_g_state <= AG_DA_TX_LOW;
        end
        
      AG_DA_TX_LOW:
        begin
        ShiftReg[31:1] <= ShiftReg[30:0];
        spi_sck <= 0;
        adc_g_bit_cnt <= adc_g_bit_cnt + 1;
        adc_g_state <= AG_DA_TX_HIGH;
        end
        
      AG_DA_TX_END1:
        begin
        spi_sck <= 0;
        spi_daccs <= 1;
        adc_g_bit_cnt <= 0;
        adc_g_state <= AG_DA_TX_ST2;
        end
        
      AG_DA_TX_END2:
        begin
        spi_sck <= 0;
        spi_daccs <= 1;
        adc_g_state <= AG_IDLE;
        end
      endcase
    end

  endmodule
